/*Info Box*/
.infoContainer {
    width: 970px;
    padding: 10px 10px 0 10px;
    margin: 0 0 10px 0;
    color: #666;
    background: #fcfcfc url(../images/bg_infobox.gif) repeat-y top right;
    border: 1px solid #b7b7b7;
    border-radius: 13px / 7px; 
    word-wrap: break-word;
    }
ul#list-files {
    float: right;
    width: 460px;
    font-size: 77%;
    color: #000;
    }
ul#list-files li {
    width: 440px;
    padding: 2px 0 2px 20px;
    margin: 0 0 2px 8px;
    background: url(../images/ico_file.gif) 2px 0 no-repeat;
    }
ul#list-files li.desc {
    margin-left: 0;
    background: url(../images/ico_description.gif) 0 0 no-repeat;
    }
#infTxt {
    float: left;
    width: 493px;
    margin: 0 0 5px 0;
    font-size:  77%;
    color: #666;
    }
/*Diff Boxes*/
.diffBox {
    width: 100%;
    padding: 6px 0;
    background: #eef6fa;
    border: 1px solid #d2d2d2;
    margin: 10px 0 0 0;
    border-radius: 5px 5px 8px 0px / 17px 17px 5px 0px;
    }
.diffBox img {
    vertical-align: middle;
    }
.diffBox a {
    text-decoration: none;
    }
.diff {
    margin: 2em 0;
    }
.diffinfo {
    overflow: hidden;
    border-bottom: 1px solid #d5d5d5;
    }
.lineNumber {
    width: 2.5%;
    border-width: 0 1px;
    border-style: solid;
    border-color: #d5d5d5;
    background: #f8fbfd;
    text-align: right;
    }
.unified .lineNumber.second {
    border-left: 0;
    }
.code,
.lineNumber {
    float: left;
    padding: 0 1%;
    color: #000;
    font-size: 85%;
    font-family: 'Ubuntu Mono', monospace;
    }
.unified .code {
    width: 87%;
    }
.sbs .code {
    width: 43%;
    overflow: hidden;
    }
.unified .insert-row .lineNumber.first {
    background: #c9ffc9;
    }
.unified .delete-row .lineNumber.second {
    background: #ffbfbf;
    }
.code.insert,
.insert-row .lineNumber.second,
.both-row .lineNumber.second {
    background: #92ed92;
    }
.code.delete,
.delete-row .lineNumber.first,
.both-row .lineNumber.first {
    background: #ff7f7f;
    }
.separate {
    height: 2em;
    background: #e3e3e3;
    }
ul#pages {
    margin-top: 10px;
    }
/*Foot Links*/
#loggerheadCont p.lpandbzr {
    float: left;
    padding: 3px 0 0 0;
    margin-top: 10px;
    }
