/*
    CSS Structure
    - Reset and Generic Styles
    - Header & Navigation
        - Loggerhead Menu Tabs
        - Sub-menu Tabs
        - Breadcrumbs
    - Search & RSS 
    - General Content
    - Log Table
    - Revision List
    - Pagination
    - Footer
*/


/* =========================
   Reset and Generic Styles
*/
body, img, form, tr, ul, li, h1, h2, h3, h4, p, table {
    padding: 0;
    margin: 0;
    }
body, html, table {
    font-size: 93%;
    }
body {
    font-weight: normal;
    font-family: Ubuntu, 'Bitstream Vera Sans', 'DejaVu Sans', Tahoma, sans-serif;
    color: #000;
    background: #fff;
    }
a {
    color: #03A;
    text-decoration: none;
    }
a:hover {
    text-decoration: underline;
    }
img {
    border: 0;
    outline: none;
    }
.clear {
    clear: both;
    font-size: 0;
    height: 0;
    }
.fr {
    float: right;
    }
.fl {
    float: left;
    }
ul {
    list-style: none;
    }
table {
    caption-side: top;
    width: 100%;
    border: none;
    border-spacing: 0;
    border-collapse: collapse;
    }
tr td {
    vertical-align: top;
    }
th, td {
    border-color: #D2D2D2;
    }
th {
    text-align: left;
    color: #5A5A5A;
    padding: 4px 8px;
    border-width: 1px;
    border-style: solid;
    }
td {
    padding: 4px 4px;
    border-width: 0 0 1px 0;
    border-style: dotted;
    }
h1 {
    font-size:138.5%;
    padding: 10px 0 5px 0;
    }
h2 {
    font-size: 123.1%;
    padding: 10px 0 5px 0;
    }
p {
    padding: 5px 0;
    }
p img {
    vertical-align: middle;
    }


/* =========================
   Header & Navigation
*/

.black-link {
    border-bottom: 1px solid #D2D2D2;
    padding: 6px 1% 8px 1%;
    font-size: 85%;
}

/* Branch Title */
h1.branch-name {
    font-size: 131%;
    padding: 0;
    margin: 20px 0 0 1%;
    }

/* Loggerhead Menu Tabs */
ul#menuTabs {
    height: 33px;
    color: #fff;
    margin: 0;
    padding: 15px 20px 0 1%;
    }
ul#menuTabs li {
    float: left;
    padding: 0 7px 0 0;
    }
ul#menuTabs li a {
    padding: 3px 6px;
    color: #000;
    }
ul#menuTabs li a:hover {
    background-color: #f3f3f3;
    text-decoration: none;
    border-radius: 5px;
    }
ul#menuTabs li a#on {
    font-weight: bold;
    color: #fff;
    background-color: #747474;
    border-radius: 5px;
    }

/* Sub-menu Tabs */
ul#submenuTabs {
    height: 24px;
    margin-bottom: 10px;
    }
ul#submenuTabs li {
    float: left;
    display: inline;
    margin-right: 20px;
    }
ul#submenuTabs li a {
    float: left;
    display: inline;
    height: 22px;
    line-height: 22px;
    padding: 0 0 0 18px;
    background: transparent url(../images/ico_description.gif) no-repeat center left;
    }

/* Breadcrumbs */
#breadcrumbs {
    font-size: 77%;
    margin-bottom: 10px;
    }

#branch-info {
    padding: 10px;
    margin: 0 0 10px 0;
    color: #666;
    background: #fcfcfc;
    border: 1px solid #b7b7b7;
    }


/* =========================
   General Content
*/
#loggerheadCont {
    width: 98%;
    padding: 0 1% 10px 1%;
    }


/* =========================
   Log Table
*/
#logentries {
    margin-bottom: 10px;
    clear: left;
    }
p.expand, p.codin {
    margin-right: 10px;
    float: left;
    }
p.expand a, p.codin a {
    color: #000;
    text-decoration: none;
    }
.show_if_js {
    display: none;
    }

/* th and td styles */
th.revisionnumber,
th.expandcell,
th.diffcell,
th.downloadcell {
    padding-left: 0;
    padding-right: 0;
    text-align: center;
    }
th.diffcell,
th.revisionnumber {
    width: 42px;
    }
th.expandcell,
td.expcell {
    width: 10px;
    }
td.icocell {
    width: 18px;
    }
th.authorcell,
td.autcell,
th.tagscell,
td.tagcell {
    width: 240px;
    }
th.datecell,
td.date {
    width: 100px;
    }
th.timedate,
td.timedate2 {
    width: 60px;
    }
th.downloadcell,
td.downr {
    width: 47px;
    }


/* =========================
   Revision List
*/
.revision_log {
    overflow: hidden;
    }
.revisioninfo {
    margin-top: 4px;
    }
ul.expandrev {
    font-size: 85%;
    }
ul.expandrev li {
    padding: 2px 0 2px 20px;
    margin: 0 0 2px 0;
    }
ul.expandrev li.mfrom {
    background: url(../images/ico_mergefrom.gif) 0 0 no-repeat;
    }
ul.expandrev li.mto {
    background: url(../images/ico_mergeto.gif) 0 0 no-repeat;
    }
ul.expandrev li.committerli {
    background: url(../images/ico_committer.gif) 2px 0 no-repeat;
    }
ul.expandrev li.desc {
    background: url(../images/ico_description.gif) 0 0 no-repeat;
    color: #666;
    }
ul.expandrev li.files {
    margin-left: 8px;
    }
li.files#file {
    background: url(../images/ico_file.gif) 2px 0 no-repeat;
}
li.files#directory {
    background: url(../images/ico_folder.png) 2px 0 no-repeat;
}
.revnolink a {
    color: #000;
    }


/* =========================
   Pagination
*/
ul#pages {
    float: right;
    }
#pages li {
    font-size: 77%;
    font-weight: bold;
    float: left;
    display: inline;
    padding: 3px 6px;
    margin-right: 2px;
    }


/* =========================
   Search & RSS
*/
#finderBox {
    width: 300px;
    float: right;
    padding: 10px 0 0 0;
    }
#finderBox label,
#finderBox input {
    float: left;
    font-weight: normal;
    font-size: 12px;
    }
#finderBox label {
    display: inline;
    width: 100px;
    text-align: right;
    color: #000;
    padding: 5px 5px 0 0;
    }
#finderBox input {
    color: #666;
    display: inline;
    width: 130px;
    padding: 3px;
    background: #fff url(../images/bg_search_input.gif) 110px 3px no-repeat;
    border: 1px solid #e9e9e9;
    }
#search_terms {
    display: none;
    border: solid 1px black;
    width: 180px;
    overflow: auto;
    background: #fff;
    }
.rssfeed {
    float: left;
    display: inline;
    padding: 0 0 0 10px;
    }


/* =========================
   Footer
*/
#footer {
    font-size: 77%;
    }

    
/* ====================
   Global notifications
*/
/* Move the content down so the notification banner doesn't hide any content. */
body.global-notification-visible {
    padding-top: 40px;
    }
body.global-notification-visible .login-logout {
    top: 45px;
    }
.notification-private {
    background-image: url(/static/images/notification-private.png); /* sprite-ref: icon-sprites */
    background-repeat: no-repeat;
    }
.global-notification {
    position: fixed;
    z-index: 10;
    top: 0;
    left: 0;
    right: 0;
    padding: 8px 20px;
    /* Define colour for browsers that don't support transparency */
    background-color: #8d1f1f;
    /* Set transparent background for browsers that support it */
    background-color: rgba(125,0,0,0.9);
    color: #fff;
    text-shadow: 0 -1px 0 #631616;
    font-size: 14px;
    line-height: 21px;
    font-weight: bold;
    -moz-box-shadow: 0 0 5px #333;
    -webkit-box-shadow: 0 0 5px #333;
    box-shadow: 0 0 5px #333;
    }
.global-notification .sprite.notification-private {
    float: left;
    display: inline-block;
    height: 21px;
    width: 20px;
    margin-right: 10px;
    padding: 0;
    }
.global-notification-close, .global-notification-close:active,
.global-notification-close:visited {
    color: #e47a7a;
    }
.global-notification-close {
    display: block;
    position: absolute;
    top: 11px;
    right: 20px;
    font-size: 12px;
    font-weight: normal;
    line-height: 14px;
    }
.global-notification-close .sprite.notification-close {
    float: right;
    display: block;
    height: 9px;
    width: 8px;
    margin: 3px 0 0 7px;
    padding: 0;
    }
